part of tecfy_basic_package;

class AppButtonPost extends StatefulWidget {
  /// button label
  final String label;

  /// button on press function
  final Future Function()? onPressed;

  /// border round edge
  final double roundEadge;

  /// button height
  final double height;

  /// background  color
  final Color? color;

  /// text style
  final TextStyle? textStyle;

  /// hero for navigation animation
  final String? heroTag;

  /// main button
  AppButtonPost(
      {required this.label,
      this.onPressed,
      this.heroTag,
      this.roundEadge = 0,
      this.height = 50.0,
      this.color,
      this.textStyle});
  @override
  _AppButtonPostState createState() => _AppButtonPostState();
}

class _AppButtonPostState extends State<AppButtonPost>
    with SingleTickerProviderStateMixin {
  double width = 320;
  AnimationController? _controller;
  Animation<double>? _animation;
  Animation<double>? _animationRadius;
  bool isBussy = false;
  @override
  initState() {
    _controller = new AnimationController(
        duration: new Duration(milliseconds: 1000), vsync: this);
    _animation = new Tween(
      begin: width,
      end: 40.0,
    ).animate(new CurvedAnimation(
        parent: _controller ?? AnimationController(vsync: this),
        curve: new Interval(0.0, 0.250)));
    _animationRadius = new Tween(
      begin: widget.roundEadge,
      end: 50.0,
    ).animate(new CurvedAnimation(
        parent: _controller ?? AnimationController(vsync: this),
        curve: new Interval(0.0, 0.250)));
    _controller?.addListener(() {
      setState(() {});
    });
    super.initState();
  }

  onPressed() async {
    try {
      isBussy = true;
      _controller?.forward();
      await widget.onPressed?.call();
      _controller?.reverse();
      isBussy = false;
    } catch (err) {}
  }

  showLoading() {
    return isBussy && (_animation?.value ?? 0) < 100;
  }

  @override
  void dispose() {
    _controller?.stop();
    _controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return widget.heroTag != null
        ? Hero(
            tag: widget.heroTag ?? '',
            child: _getButton()) //"submit_button" + widget.label
        : _getButton();
  }

  Widget _getButton() {
    return MaterialButton(
      shape: RoundedRectangleBorder(
          borderRadius: BorderRadius.circular(_animationRadius?.value ?? 0)),
      color: widget.color ?? ThemeService.secondryColor,
      onPressed: onPressed,
      child: Container(
        height: widget.height,
        width: _animation?.value == width
            ? null
            : _animation?.value, //double.infinity,
        child: Center(
          child: showLoading()
              ? CircularProgressIndicator(
                  valueColor: AlwaysStoppedAnimation<Color>(Colors.white),
                )
              : Text(
                  widget.label,
                  style: widget.textStyle ??
                      TextStyle(
                          color: ThemeService.textOnSecondColor,
                          fontSize: 18.0),
                ),
        ),
      ),
    );
  }
}
